﻿using System;
using System.Collections.Concurrent;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;
using Voice_IVR.Model;
using Voice_IVR.Model.InputInterpreter;
using Voice_IVR.Model.IVRMenuElements;
using Voice_IVR.Windows;
using System.Collections.Generic;

namespace Voice_IVR
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private void Tsmi_HelpPage_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void Tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();

            aboutBox.ShowDialog(this);
        }

        private IVREngine ivrEngine;


        private void FormMain_Load(object sender, EventArgs e)
        {
            Tssl_Ozeki.Text = string.Format("© Copyright {0}, Ozeki Ltd.", DateTime.Now.Year);

            ISoftPhone softPhone = null;

            /* Create a softphone...
             * */
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5000, 8000, 5060);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, @"Please read the DTMF IVR manual.");

                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if(ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
				
                return;
            }

            /* Set the SIP Account settings..
             * */
            SIPAccount acc = new SIPAccount(
                   registrationRequired: true,
                   displayName: "oz875",
                   userName: "oz875",
                   registerName: "oz875",
                   registerPassword: "oz875",
                   domainServerHost: "192.168.112.100",
                   domainServerPort: 5060
               );

            Dictionary<IPhoneCall, CallSoundEngine> soundEngines = new Dictionary<IPhoneCall, CallSoundEngine>();

            /* Create IVRMenu
             * */
            IIVRMenuElement rootMenu = CreateIVRMenu(soundEngines);

            /* Create IVREngine
             * */
            ivrEngine = new IVREngine(soundEngines, softPhone, rootMenu, acc);


            IVREngine.NewEngineEvent += IVREngine_NewReportEvent;

            /* Start IVREngine
             * */
            ivrEngine.Start();
        }

        private IIVRMenuElement CreateIVRMenu(Dictionary<IPhoneCall, CallSoundEngine> soundEngines)
        {
            VoiceMessageData voiceMessageData = new VoiceMessageData("name@example.hu", "name@example.hu", "subject", "body");
            string smtpServer = "example.com";

            /*Create IVR root menu
             */
            IIVRMenuElement rootMenu = new WavePlayerElement(
                soundEngines, "Resources\\demo.wav", new SpeechToTextInputInterpreter(soundEngines,
                    new[] { "Email", "Speech", "Softphone" }
                )
            );

            /*Create IVR submenus
             */
            IIVRMenuElement dataBaseTextToSpeechElement = new DataBaseTextToSpeechElement(
                soundEngines, new SpeechToTextInputInterpreter(soundEngines, new[] { "end" }));
            
            IIVRMenuElement voiceMessageToEmailElement = new VoiceMessageToEmailElement(
                soundEngines, smtpServer, voiceMessageData, null,
                new SpeechToTextInputInterpreter(soundEngines, new[] { "end" })
            );

            IIVRMenuElement softPhoneElement = new SoftPhoneElement(soundEngines,
                new SpeechToTextInputInterpreter(soundEngines, new[] { "end" })
            );

            /*Adding submenus to the root menu
             */
            rootMenu.AddSubMenuElement("Email", voiceMessageToEmailElement);
            rootMenu.AddSubMenuElement("Speech", dataBaseTextToSpeechElement);
            rootMenu.AddSubMenuElement("Softphone", softPhoneElement);

            /*Create IVRMenu end element
             */
            IIVRMenuElement endMenuElement = new EndElement(soundEngines);

            /*Adding end menu to the submenus
             */
            rootMenu.AddSubMenuElement("end", endMenuElement);
            dataBaseTextToSpeechElement.AddSubMenuElement("end", endMenuElement);
            voiceMessageToEmailElement.AddSubMenuElement("end", endMenuElement);
            softPhoneElement.AddSubMenuElement("end", endMenuElement);

            Lb_Report.Items.Add("---------- This sample IVR system works by voice control that means you can navigate the IVR tree with spoken words. -------------" + Environment.NewLine);
            Lb_Report.Items.Add("-------------------------");
            Lb_Report.Items.Add("Sample IVR menu stucture:");
            Lb_Report.Items.Add("----------");
            Lb_Report.Items.Add("root waveplayback menu (demo.wav playback)");
            Lb_Report.Items.Add("  'email'- voice message to email menu");
            Lb_Report.Items.Add("          'exit' - exit");
            Lb_Report.Items.Add("  'speech'- databese textToSpeech menu");
            Lb_Report.Items.Add("           'exit' - exit");
            Lb_Report.Items.Add("  'softphone' - softphone menu");
            Lb_Report.Items.Add("           'exit' - exit");
            Lb_Report.Items.Add("-------------------------");
            return rootMenu;
        }

        void IVREngine_NewReportEvent(object sender, EngineInformation e)
        {
            BeginInvoke((Action)(() => Lb_Report.Items.Add(e.EngineElementName + " - " + e.CallId + " - " + e.ReportString)));
        }

        private void bt_Start_Click(object sender, EventArgs e)
        {
            ivrEngine.Start();

            bt_Stop.Enabled = true;
            bt_Start.Enabled = false;
        }

        private void bt_Stop_Click(object sender, EventArgs e)
        {
            ivrEngine.Stop();

            bt_Stop.Enabled = false;
            bt_Start.Enabled = true;
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            ivrEngine.Stop();
        }
    }
}
