﻿using System;
using System.Collections.Concurrent;
using System.Threading;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;
using Voice_IVR.Model.IVRMenuElements;
using System.Collections.Generic;

namespace Voice_IVR.Model
{

    #region EngineInformation

    internal class EngineInformation : EventArgs
    {
        public EngineInformation(string engineElementName, string callId, string reportString)
        {
            CallId = callId;
            EngineElementName = engineElementName;
            ReportString = reportString;
        }

        public string CallId { get; set; }
        public string ReportString { get; set; }
        public string EngineElementName { get; set; }
    }

    #endregion

    internal class IVREngine
    {
        private readonly IIVRMenuElement baseIVRMenuElement;
        private IPhoneLine line;
        public Dictionary<IPhoneCall, CallSoundEngine> ActiveCallSoundEngines { get; private set; }
        public ISoftPhone SoftPhone { get; set; }
        public SIPAccount Acc { get; set; }
        private bool registerSucceeded;

        public static event EventHandler<EngineInformation> NewEngineEvent;

        /// <summary>
        /// Create the IVREngine.
        /// </summary>
        /// <param name="softPhone">Ozeki VoIP SIP SDK softphone</param>
        /// <param name="menuElement"></param>
        /// <param name="acc"></param>
        internal IVREngine(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines, 
            ISoftPhone softPhone, 
            IIVRMenuElement menuElement, 
            SIPAccount acc)
        {
            SoftPhone = softPhone;
            SoftPhone.IncomingCall += SoftPhone_IncomingCall;
            Acc = acc;
            baseIVRMenuElement = menuElement;
            ActiveCallSoundEngines = soundEngines;
            registerSucceeded = false;
        }

        /// <summary>
        /// Activates the EngineEvent.
        /// </summary>
        /// <param name="elementName"></param>
        /// <param name="callID"></param>
        /// <param name="reportString"></param>
        public static void OnEngineEvent(string elementName, string callID, string reportString)
        {
            if (NewEngineEvent != null)
                NewEngineEvent(null, new EngineInformation(elementName, callID, reportString));
        }


        /// <summary>
        /// Start the Engine.
        /// </summary>
        public void Start()
        {
            OnEngineEvent(string.Format("- - - Tries to register {0} SIP account.", Acc.SIPURL), "", "");
            line = SoftPhone.CreatePhoneLine(Acc,new NatConfiguration(NatTraversalMethod.None,""));
            SoftPhone.RegisterPhoneLine(line);
            OnEngineEvent("- - - IVR service start", "-", "");
        }


        /// <summary>
        /// Handles the events IncomingCall.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SoftPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            IPhoneCall call = e.Item;
            CallSoundEngine soundEngine = new CallSoundEngine(call);
            ActiveCallSoundEngines.Add(call, soundEngine);
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.Accept();
            
            IIVRMenuElement ownMenu = (IIVRMenuElement)baseIVRMenuElement.Clone();
            new Thread(() => ownMenu.EngineExecute(e.Item)).Start();
        }


        void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            IPhoneCall call = sender as IPhoneCall;
            if (call == null)
                return;

            if (call.CallState > CallState.InCall)
            {
                call.CallStateChanged -= (call_CallStateChanged);
            }
        }

        /// <summary>
        /// Stop the Engine.
        /// </summary>
        internal void Stop()
        {
            SoftPhone.UnregisterPhoneLine(line);
            OnEngineEvent("- - - IVR service stop", "-", "");
        }
    }
}